module.exports = function(grunt) {

    var srcFiles = {
        templates: ['approval_components/js/controllers/*.js', 'approval_components/js/directives/*.js'],
        js: ['js/utilities/*.js', 'js/repositories/*.js', 'js/services/*.js', 'js/controllers/*.js']
    };

    grunt.initConfig({
        less: {
            development: {
                options: {
                    compress: true
                },
                files: {
                    'css/css-compile.css': "css/import-ordering.less"
                }
            }
        },
        concat: {
            templates: {
                options: {
                    process: function(src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: srcFiles.templates,
                dest: 'dist/templates.js'
            },
            js: {
                options: {
                    process: function(src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: srcFiles.js,
                dest: 'dist/app.js'
            }
        },
        jsbeautifier: {
            final: {
                src: ['Gruntfile.js', srcFiles.js],
                options: {
                    js: {
                        maxPreserveNewlines: 0,
                        preserveNewlines: false
                    }
                }
            },
            dev: {
                src: ['Gruntfile.js', srcFiles.js],
                options: {
                    js: {
                        maxPreserveNewlines: 2
                    }
                }
            }
        },
        watch: {
            js: {
                files: srcFiles.js,
                tasks: ['jsbeautifier:dev', 'concat:js', 'buildEnvironment']
            },
            css: {
                files: srcFiles.css,
                tasks: ['concat:css']
            }
        },
        jshint: {
            merge: {
                src: ['Gruntfile.js', srcFiles.js],
                "options": {
                    reporter: require('jshint-html-reporter'),
                    reporterOutput: './logs/dev.html',
                    esversion: 6
                }
            },
            dev: {
                src: ['Gruntfile.js', srcFiles.js],
                "options": {
                    reporter: require('jshint-html-reporter'),
                    reporterOutput: './logs/dev.html',
                    esversion: 6
                }
            }
        },
        buildEnvironment: {
            deployHomo: {
                configFile: "config/deployHomo.json"
            },
            development: {
                configFile: "config/development.json"
            },
            deploy: {
                configFile: "config/deploy.json"
            },

            test: {
                configFile: "config/test.json"
            },
            srcFolders: ["./js/", "./config", "./", "./", "./json", "./json/words"]
        },
        zhWords: {
            options: {
                serverUrl: "https://rc-translate.teknisa.com/backend/index.php",
                organization: "20",
                project: "27258",
                repository: "1601",
                wordsPath: "./json/words"
            }
        }
    });

    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks("grunt-jsbeautifier");
    grunt.loadNpmTasks('grunt-contrib-less');
    grunt.loadNpmTasks("grunt-notify");
    grunt.loadNpmTasks('grunt-zh-words');
    // Default task.
    grunt.registerTask('default', ['less', 'jsbeautifier:dev', 'jshint:dev', 'concat', 'zhWords']);
    // Watch task.
    grunt.registerTask('devCoPilot', ['watch', 'notify']);
    // Other tasks.
    grunt.registerTask('merge', ['jsbeautifier:final', 'jshint:merge', 'concat']);
    grunt.registerTask('dev', ['jsbeautifier:dev', 'jshint:dev', 'concat']);
    grunt.registerTask('prod', ['jsbeautifier:final', 'concat']);
    grunt.registerTask('test', ['jsbeautifier:final', 'concat']);
};
