function ApprovalFilterController($scope, templateManager, ScreenService, ApplicationContext) {

	$scope.filterValues = {};
	$scope.fieldValues = {};
	let filterBackup;
	let dataSourceFilterBackup;
	
	$scope.childrenScopeInterface.createBackup = function(){
		let dataSourceFilter = templateManager.container.getActiveWidget().dataSourceFilter;
		dataSourceFilterBackup = dataSourceFilter.map(filter => Object.assign({}, filter));
		filterBackup = Object.assign({}, $scope.filterValues);
	};
	
	$scope.childrenScopeInterface.restoreBackup = function() {
		templateManager.container.getActiveWidget().dataSourceFilter = dataSourceFilterBackup;
		// attributing filterValues one by one to trigger setters
		for (const filterElementName in filterBackup) {
			$scope.filterValues[filterElementName] = filterBackup[filterElementName];
		}
	};
	
	// @Deprecated
	function buildDateFilter(dataSourceFilter, dateFieldValue) {
		if(dateFieldValue === "") {
			if(dataSourceFilter.DTINICIAL){
				delete dataSourceFilter.DTINICIAL;
			}
			if(dataSourceFilter.DTFINAL){
				delete dataSourceFilter.DTFINAL;
			}
			dataSourceFilter.TDATAS = "T";
		}
	}
	
	// @Deprecated
	function buildFilterOld(widget){
		let valuesToConcatInDataSourceFilter = Object.assign({}, $scope.filterValues);
		
		let dateFieldValue = valuesToConcatInDataSourceFilter.dateFieldValueField;
		delete valuesToConcatInDataSourceFilter.dateFieldValueField;
		buildDateFilter(widget.dataSourceFilter, dateFieldValue);
		
		// overlap existing properties	
		widget.dataSourceFilter.forEach(filterValue => {
			
			if(valuesToConcatInDataSourceFilter.hasOwnProperty(filterValue.name)) {
				
				filterValue.value = valuesToConcatInDataSourceFilter[filterValue.name];
				if(filterValue.value === "") {
					filterValue.value = "T";
				}
				
				delete valuesToConcatInDataSourceFilter[filterValue.name];
			} 
			
		});
		
		// concat properties that don't exist yet
		for (let filterNmae in valuesToConcatInDataSourceFilter) {
			let valueToConcat = valuesToConcatInDataSourceFilter[filterNmae];
			if(valueToConcat === "") {
				valueToConcat = 'T';	
			}
			widget.dataSourceFilter.push({
				name: filterNmae,	
				operator: "=",	
				value: valueToConcat,	
			});
		}
		
	}
	function buildFilter(widget){
		let valuesToConcatInDataSourceFilter = Object.assign({}, $scope.filterValues);
		let fields = templateManager.container.getActiveWidget().filterProperties.fields;
		fields.forEach(f => {
			if(f.skipAutoFilter) 
				delete valuesToConcatInDataSourceFilter[f.name];
		});
		
		for (let filterNmae in valuesToConcatInDataSourceFilter) {
			let valueToConcat = valuesToConcatInDataSourceFilter[filterNmae];
			if(valueToConcat === "") {
				valueToConcat = 'T';	
			}
			let filterToChange = widget.dataSourceFilter.find(filter => filter.name === filterNmae);
			if(filterToChange) {
				filterToChange.value = 	valueToConcat;
			} else {
				widget.dataSourceFilter.push({
					name: filterNmae,	
					operator: "=",	
					value: valueToConcat,	
				});
			}
		}
		
	}
	
	$scope.applyFilter = function() {
		let activeWidget = templateManager.container.getActiveWidget();
		if(activeWidget.filterProperties.beforeFilterApply){
			evaluateExpression(activeWidget.filterProperties.beforeFilterApply, 
				Object.assign({}, ApplicationContext), true);
		}
		buildFilter(activeWidget);
		return activeWidget.reload();
	};	
	
	// @Deprecated
	$scope.onValueSelected = function (fieldValue, fieldName) {
		let activeWidget = templateManager.container.getActiveWidget();
		if(!activeWidget.filterValue) {
			activeWidget.filterValue = {};
		}
		activeWidget.filterValue[fieldName] = fieldValue;
	};
	
	function evaluateExpression(expression, context, withoutAtChar) {
		if(withoutAtChar || expression.match(/\@(.*?)\@/g)) {
			expression = expression.replace(/\@/g, '');
			return $scope.$eval(expression, context);
		} else {
			return expression;
		}
	}
	
	let reload = function(){
		this.aditionalOptions = this.aditionalOptions || [];
		ApplicationContext.RestEngine.request(this.dataSource.name, {
			requestType: "FilterData",
			itemsPerPage: 500,
			page: 1,
		    filter: this.dataSourceFilter || []
		}).then(response => {
		        $scope.fieldValues[this.name] = this.aditionalOptions.concat(response.dataset.filial);
		});
	};
	
	$scope.initFields = function(){
		const activeWidget = templateManager.container.getActiveWidget();
		activeWidget.filterValues = $scope.filterValues;
		let fields = activeWidget.filterProperties.fields;
		for (let i = 0; i < fields.length; i++) {
			let defaultValue = fields[i].default;
			if(defaultValue) {
				defaultValue = evaluateExpression(defaultValue);
				$scope.filterValues[fields[i].name] = defaultValue;
			}
			if(fields[i].events) {
				let fieldOnChangeEvent = fields[i].events.find((e) => e.name === 'FieldOnChange');
				if(fieldOnChangeEvent && fieldOnChangeEvent.code) {
					fields[i].onchange = fieldOnChangeEvent.code;
				}
			}
			if(fields[i].type === undefined || fields[i].type === 'select'){
				if(fields[i].dataSource && fields[i].dataSource.rest) {
					fields[i].reload = reload.bind(fields[i]);
					fields[i].reload();
				} else {
					$scope.fieldValues[fields[i].name] = fields[i].dataSource.data;
				}
			} else if(fields[i].type === 'toggle') {
				fields[i].valueToTrue = fields[i].valueToTrue || true; 
				fields[i].valueTofalse = fields[i].valueTofalse || false; 
			}
		}
		
		/**
		 * Usar quando o lazyLoad do dataSource da widget ativa estiver true.
		 * Força a realizar o filtro com um dataSourceFilter modificado de
		 * acordo com o filtro atual carregado.
		 */
		if (activeWidget.applyFilterOnInit) {
			$scope.applyFilter();
		}
	}
	
	$scope.triggerOnChange = function(field) {
		if(field.onchange) {
			let activeWidget = templateManager.container.getActiveWidget();
			evaluateExpression(
				field.onchange, 
				Object.assign(ApplicationContext, {
					filterValues: $scope.filterValues,
					currentWidget: activeWidget
				}), 
				true
			);
		}
	};
	
	$scope.initTest = function(){
		getData();	
	};
	
	$scope.beforeSelectAutoCompleteInitialized = (autoCompleteScope) => {
		/**
		 * the select auto complete field definition is not inside a widget,
		 * so we need to factor a dataSource by our own.
		 * :/
		 */
		autoCompleteScope.field.dataSource = metaDataFactory.dataSourceFactory(
			autoCompleteScope.field.dataSource,
			autoCompleteScope.field.dataSourceFilter,
			autoCompleteScope.field
		);
	};

	$scope.onSelectAutoCompleteInitialized = (autoCompleteScope) => {
		autoCompleteScope.__fieldRow = $scope.filterValues[autoCompleteScope.field.name];
        Object.defineProperty($scope.filterValues, autoCompleteScope.field.name, {
            get: function() {
                return autoCompleteScope.__fieldRow;
            },
            set: function(newValue) {
                autoCompleteScope.__fieldRow = newValue;
				// autoCompleteScope.widget.editRow[autoCompleteScope.field.name] = newValue;
				let selectedElement = autoCompleteScope.field.dataSource.data.find(dataSourceElement => {
					return dataSourceElement[autoCompleteScope.field.name] === autoCompleteScope.__fieldRow;
				});
				if(selectedElement){
					autoCompleteScope.descriptionText = autoCompleteScope.getDescriptionString(selectedElement);
				}
            },
            enumerable: true,
            configurable: true
		});
		// compute initial descrption text
		if(!autoCompleteScope.descriptionText && autoCompleteScope.__fieldRow){
			let selectedElement = autoCompleteScope.field.dataSource.data.find(dataSourceElement => {
				return dataSourceElement[autoCompleteScope.field.name] === autoCompleteScope.__fieldRow;
			});
			if(selectedElement){
				autoCompleteScope.descriptionText = autoCompleteScope.getDescriptionString(selectedElement);
			}
			
		}
	};
}