var TreeController = (function(_super) {
	__extends(TreeController1, _super);

	function TreeController1($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query, CrosstabService, $filter, maskEngine, ZhUtil, $window, ApplicationContext, ZhFeatures, eventAggregator) {
		var _ref = TreeController.__super__.constructor.apply(this, arguments);

		// BEGIN: Initialize
		var self = this;
		var CONTROLLER_KEY = 'treeControllerInit';
		var ALL_DATA = 999999999;

		var methodsToLoad = {
			"afterLoadDataSource": function() {
				reloadTreeDataSource();
			}
		};

		Util.initializeComponent(this.widget, CONTROLLER_KEY, methodsToLoad, undefined, eventEngine);
		// END: Initialize

		// BEGIN: Tree
		var parentProperty = this.widget.parentProperty;
		var primaryProperty = this.widget.primaryProperty;
		var breadcrumbProperty = this.widget.breadcrumbProperty;
		var treeChildrenMap = {};

		// Floating Control defaults
		var floatingControlDefaults = {
			pdfAction: true,
			xlsAction: true,
			csvAction: true,
			customizationAction: {
				filter: true,
				layout: false,
				view: false
			}
		};
		self.updateFloatingControlDefaults(floatingControlDefaults);

		function _buildTreeData(data, childKey, parentKey, parentValue, level, parent) {
			var ignoreChildren = false;

			var dataResult = data.filter(function(item) {
				return item[childKey] === parentValue || (parentValue === null && !item[childKey]);
			});

			if(dataResult.length === 0 && data.length && !parent){
				ignoreChildren = true;
				dataResult = data;
			}

			dataResult.forEach(function(item) {
				item._parent = parent;
				// generate the breadcrumb for the item
				if (parent && breadcrumbProperty) {
					item._breadcrumb = (parent._breadcrumb ? parent._breadcrumb + ' > ' : '') + parent[breadcrumbProperty];
				}
				if (item[parentKey]) {
					item._children = _buildTreeData(data, childKey, parentKey, item[parentKey], level + 1, item);
					if(ignoreChildren){
						item._children = [];
					}
				} else {
					item._children = [];
				}

				item._level = level;
			});

			return dataResult;
		}

		function plain(data, result) {
			data.forEach(function(child) {
				var lastIndex = result.push(child);
				result[lastIndex - 1]._childCount = result[lastIndex - 1]._children.length;
				plain(child._children, result);

				treeChildrenMap[result[lastIndex - 1][primaryProperty]] = result[lastIndex - 1]._children;
				delete result[lastIndex - 1]._children;
			});
		}

		function reloadTreeDataSource() {
			if (self.isTree(self.widget) || self.isGantt(self.widget)) {
				self.widget.treeData = [];
				if (self.widget.dataSource && self.widget.dataSource.data) {
					treeChildrenMap = {};
					self.widget.treeData = _buildTreeData(self.widget.dataSource.data, parentProperty, primaryProperty, null, 1, null);
					self.widget.plainTreeData = [];
					plain(self.widget.treeData, self.widget.plainTreeData);
					if(self.widget.isOpened){
						self.widget.openAllGrid();
					}
				}
			}
		}

		this.handleDependentRows = function(row, widget){
			handleSelectCascade(row, widget);
			childrenDependencyApply(row, widget);
		};

		function _selectCascadeApply(row, widget) {
			if (row && Util.isArray(treeChildrenMap[row[primaryProperty]])) {
				treeChildrenMap[row[primaryProperty]].forEach(function(child) {
					child.__isSelected = !!row.__isSelected;
					if(!child.__isSelected){
						widget.dataSource.removeCheckedRows(child);
					}
					_selectCascadeApply(child, widget);
				});
			}
		}

		function handleSelectCascade(row, widget){
			if (widget.selectCascade) {
				_selectCascadeApply(row, widget);
				childrenDependencyApply(row, widget);
			}
		}

		function childrenDependencyApply(row, widget){
			if(widget.childrenDependency){
				if(row && row._parent){
					var childrenSelect = isEveryChildSelected(row._parent);
					if (childrenSelect !== row._parent.__isSelected) {
						row._parent.__isSelected = childrenSelect;
						allowDeselect(row, widget);
						childrenDependencyApply(row._parent, widget);
					}
				}
				allowDeselect(row, widget);
			}
		}

		function allowDeselect(row, widget){
			var children = treeChildrenMap[row[primaryProperty]];
			if(widget.childrenDependency &&  !widget.selectCascade && Util.isArray(children) && children.length){
				row.__isSelected = isEveryChildSelected(row);
			}
		}

		function isEveryChildSelected(parent){
			if (parent && Util.isArray(treeChildrenMap[parent[primaryProperty]])) {
				return treeChildrenMap[parent[primaryProperty]].every(function(child){
					return child.__isSelected && isEveryChildSelected(child);
				});
			}
			return false;
		}

		function openOrCloseGrid(opened){
			self.widget.plainTreeData.forEach(function(row){
				if(row._childCount && row._childCount > 0){
						row._opened = opened;
				}
			});
		}

		if (this.isTree(self.widget) || this.isGantt(self.widget)) {
			self.widget.openAllGrid = function(){
				openOrCloseGrid(true);
			};

			self.widget.closeAllGrid = function(){
				openOrCloseGrid(false);
			};

			$scope.$watch('widget', reloadTreeDataSource);

			$scope.$watchCollection('widget.dataSource.data', reloadTreeDataSource);
		}

		this.getRowLevel = function(row, search) {
			return !Util.objectHasAnyValue(search) ? row._level : 1;
		};

		this.fetchOnDemand = function(row, widget) {
			if (widget.onDemand && !row.__processed) {
				row.__processed = true;
				var filters = widget.getDataSourceFilter();
				filters = filters.concat(Query.build().where(parentProperty).equals(row[primaryProperty] || "").where());
				var oldData = widget.dataSource.data || [];
				widget.dataSource.load(filters, 1, ALL_DATA, function(data) {
					widget.dataSource.data = oldData.concat(data);
					templateManager.updateTemplate();
					if (widget.resizeGridColumns) { // new tree
						widget.resizeGridColumns(true);
					}
				});
			}
		};

		this.parentIconContent = function(row) {
			if (row._hasChild || row._childCount > 0) {
				if (row._opened) {
					return "opened";
				} else {
					return "closed";
				}
			} else {
				return "no-child";
			}
		};

		this.isOpened = function(row) {
			if (row._parent === null) {
				return row._opened;
			} else {
				return row._parent._opened && this.isOpened(row._parent);
			}
		};

		// Overwrite of widget.reload to keep tree opened
		if (!this.widget.innerReload) {
			this.widget.innerReload = this.widget.reload;
		}

		this.widget.reload = function(addictionalFilters) {
			if (self.isTree(self.widget) && this.primaryProperty && !this.onDemand) {
				var oldDataSet = {};
				this.dataSource.data.forEach(function(row) {
					oldDataSet[row[this.primaryProperty]] = !!row._opened;
				}.bind(this));

				return this.innerReload(addictionalFilters).then(function(data) {
					this.dataSource.data.forEach(function(row) {
						if(oldDataSet.hasOwnProperty(row[this.primaryProperty])){
							row._opened = !!oldDataSet[row[this.primaryProperty]];
						}
					}.bind(this));
				}.bind(this));
			} else {
				return this.innerReload(addictionalFilters);
			}
		};

		this.repeatFilter = function(item) {
			return self.search(item) && (self.isOpened(item) || item._parent === null || Util.objectHasAnyValue($scope.widget.searchCriteria));
		};

		$scope.levelRange = function(level) {
			var result = [];
			for (var i = 1; i < level; i++) {
				result.push(i);
			}

			return result;
		};
		// END: Tree

		return _ref;
	}

	return TreeController1;
})(GridController);